for x in *a, *b:
    print(x)

for x in a, b, *c:
    print(x)

for x in *a, b, c:
    print(x)

for x in *a, b, *c:
    print(x)

async for x in *a, *b:
    print(x)

async for x in *a, b, *c:
    print(x)

async for x in a, b, *c:
    print(x)

async for x in (
    *loooooooooooooooooooooong,
    very,
    *loooooooooooooooooooooooooooooooooooooooooooooooong,
):
    print(x)
